#ifndef MESHDISPLAYLIST_H
#define MESHDISPLAYLIST_H

#include "mesh.h"
#include "fw.h"

class MeshDisplayList : public Mesh
{
	public:

		MeshDisplayList(void) : dl(0) { }

		virtual void create(const Builder & b)
		{
			Mesh::create(b);
			this->dl = glGenLists(1);
			this->fillDisplayList();
		}

		virtual void destroy(void)
		{
			glDeleteLists(this->dl, 1);
			this->dl = 0;
			Mesh::destroy();
		}

		virtual void update(const Updater & u, float dt)
		{
			Mesh::update(u, dt);
			this->fillDisplayList();
		}

		virtual void render(void) const
		{
			glCallList(this->dl);
		}

	protected:

		GLuint dl;

		void fillDisplayList(void)
		{
			glNewList(this->dl, GL_COMPILE);
				// send vertices as in imediate mode
				glBegin(GL_TRIANGLES);
					for (size_t i=0; i<this->triangles.size(); ++i)
					{
						const Triangle & t = this->triangles[i];
						for (int j=0; j<3; ++j)
						{
							const unsigned int index = t.vertices[j];
							const Vertex & v = this->vertices[index];
							glNormal3fv   (v.normal);
							glColor4ubv   (v.color);
							glTexCoord2fv (v.texcoord);
							glVertex3fv   (v.position);
						}
					}
				glEnd();
			glEndList();
		}
};

#endif // MESHDISPLAYLIST_H
