#ifndef MESHVERTEXARRAYOBJECT_H
#define MESHVERTEXARRAYOBJECT_H

#include "mesh.h"
#include "fw.h"

class MeshVertexArrayObject : public Mesh
{
	public:

		MeshVertexArrayObject(void) : vbo(0), ibo(0), vao(0) { }

		virtual void create(const Builder & b)
		{
			Mesh::create(b);

			glGenBuffers(1, &(this->vbo));
			glBindBuffer(GL_ARRAY_BUFFER, this->vbo);
			glBufferData(GL_ARRAY_BUFFER, this->vertices.size() * sizeof(Vertex), &(this->vertices[0]), GL_DYNAMIC_DRAW);
			glBindBuffer(GL_ARRAY_BUFFER, 0);

			glGenBuffers(1, &(this->ibo));
			glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, this->ibo);
			glBufferData(GL_ELEMENT_ARRAY_BUFFER, this->triangles.size() * sizeof(Triangle), &(this->triangles[0]), GL_DYNAMIC_DRAW);
			glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

			glGenVertexArrays(1, &(this->vao));
			glBindVertexArray(this->vao);

			const GLsizei stride = sizeof(Vertex);
			const unsigned char * vbase = reinterpret_cast<const unsigned char *>(0);

			glBindBuffer(GL_ARRAY_BUFFER, this->vbo);
#if USE_GENERIC_VERTEX_ATTRIBS
			glVertexAttribPointer(0, 3, GL_FLOAT,         GL_FALSE, stride, vbase + VERTEX_FIELD_BYTE_OFFSET(position));
			glVertexAttribPointer(1, 3, GL_FLOAT,         GL_FALSE, stride, vbase + VERTEX_FIELD_BYTE_OFFSET(normal));
			glVertexAttribPointer(2, 2, GL_FLOAT,         GL_FALSE, stride, vbase + VERTEX_FIELD_BYTE_OFFSET(texcoord));
			glVertexAttribPointer(3, 4, GL_UNSIGNED_BYTE, GL_TRUE,  stride, vbase + VERTEX_FIELD_BYTE_OFFSET(color));

			glEnableVertexAttribArray(0);
			glEnableVertexAttribArray(1);
			glEnableVertexAttribArray(2);
			glEnableVertexAttribArray(3);
#else
			glVertexPointer   (3, GL_FLOAT,         stride, vbase + VERTEX_FIELD_BYTE_OFFSET(position));
			glNormalPointer   (   GL_FLOAT,         stride, vbase + VERTEX_FIELD_BYTE_OFFSET(normal)  );
			glTexCoordPointer (2, GL_FLOAT,         stride, vbase + VERTEX_FIELD_BYTE_OFFSET(texcoord));
			glColorPointer    (4, GL_UNSIGNED_BYTE, stride, vbase + VERTEX_FIELD_BYTE_OFFSET(color)   );

			glEnableClientState(GL_VERTEX_ARRAY       );
			glEnableClientState(GL_NORMAL_ARRAY       );
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
			glEnableClientState(GL_COLOR_ARRAY        );
#endif
			glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

			glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, this->ibo);

			glBindVertexArray(0);

			glBindBuffer(GL_ARRAY_BUFFER, 0);
		}

		virtual void destroy(void)
		{
			glDeleteBuffers(1, &(this->vbo));
			this->vbo = 0;

			glDeleteBuffers(1, &(this->ibo));
			this->ibo = 0;

			glDeleteVertexArrays(1, &(this->vao));
			this->vao = 0;

			Mesh::destroy();
		}

		virtual void update(const Updater & u, float dt)
		{
			Mesh::update(u, dt);

			glBindBuffer(GL_ARRAY_BUFFER, this->vbo);
			glBufferData(GL_ARRAY_BUFFER, this->vertices.size() * sizeof(Vertex), &(this->vertices[0]), GL_DYNAMIC_DRAW);
			glBindBuffer(GL_ARRAY_BUFFER, 0);

			//glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, this->ibo);
			//glBufferData(GL_ELEMENT_ARRAY_BUFFER, this->triangles.size() * sizeof(Triangle), &(this->triangles[0]), GL_DYNAMIC_DRAW);
			//glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
		}

		virtual void render(void) const
		{
			if (this->vertices.empty() || this->triangles.empty()) return;

			const unsigned char * ibase = reinterpret_cast<const unsigned char *>(0);
			const unsigned int firstTriangle = 0;

			glBindVertexArray(this->vao);
				glDrawElements(GL_TRIANGLES, this->triangles.size() * 3, GL_UNSIGNED_INT, ibase + firstTriangle * sizeof(Triangle));
			glBindVertexArray(0);
		}

	protected:

		GLuint vbo;
		GLuint ibo;
		GLuint vao;
};

#endif // MESHVERTEXARRAYOBJECT_H
